package ru.dbotthepony.kstarbound.client.network.packets

import io.netty.buffer.ByteBuf
import ru.dbotthepony.kstarbound.client.network.ClientConnection
import ru.dbotthepony.kstarbound.client.world.ClientWorld
import ru.dbotthepony.kstarbound.network.IClientPacket
import ru.dbotthepony.kstarbound.io.readUUID
import ru.dbotthepony.kstarbound.io.readVec2i
import ru.dbotthepony.kstarbound.io.writeUUID
import ru.dbotthepony.kstarbound.io.writeVec2i
import ru.dbotthepony.kstarbound.world.World
import ru.dbotthepony.kstarbound.world.WorldGeometry
import ru.dbotthepony.kvector.vector.Vector2i
import java.io.DataInputStream
import java.io.DataOutputStream
import java.util.UUID

data class JoinWorldPacket(val uuid: UUID, val seed: Long, val geometry: WorldGeometry) : IClientPacket {
	constructor(buff: DataInputStream) : this(buff.readUUID(), buff.readLong(), WorldGeometry(buff))
	constructor(world: World<*, *>) : this(UUID(0L, 0L), world.seed, world.geometry)

	override fun write(stream: DataOutputStream) {
		stream.writeUUID(uuid)
		stream.writeLong(seed)
		geometry.write(stream)
	}

	override fun play(connection: ClientConnection) {
		connection.client.world = ClientWorld(connection.client, seed, geometry)
	}
}
