package ru.dbotthepony.kstarbound.server.world

import org.apache.logging.log4j.LogManager
import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.io.BTreeDB
import ru.dbotthepony.kstarbound.io.readVarInt
import ru.dbotthepony.kstarbound.json.VersionedJson
import ru.dbotthepony.kstarbound.util.KOptional
import ru.dbotthepony.kstarbound.world.CHUNK_SIZE
import ru.dbotthepony.kstarbound.world.ChunkPos
import ru.dbotthepony.kstarbound.world.api.AbstractCell
import ru.dbotthepony.kstarbound.world.api.MutableCell
import ru.dbotthepony.kstarbound.world.entities.WorldObject
import ru.dbotthepony.kvector.arrays.Object2DArray
import ru.dbotthepony.kvector.vector.Vector2i
import java.io.BufferedInputStream
import java.io.ByteArrayInputStream
import java.io.DataInputStream
import java.util.concurrent.CompletableFuture
import java.util.zip.Inflater
import java.util.zip.InflaterInputStream

class LegacyChunkSource(val db: BTreeDB) : IChunkSource {
	override fun getTiles(pos: ChunkPos): CompletableFuture<KOptional<Object2DArray<out AbstractCell>>> {
		return CompletableFuture.supplyAsync {
			val chunkX = pos.x
			val chunkY = pos.y
			val key = byteArrayOf(1, (chunkX shr 8).toByte(), chunkX.toByte(), (chunkY shr 8).toByte(), chunkY.toByte())
			val data = db.read(key) ?: return@supplyAsync KOptional.empty()

			val reader = DataInputStream(BufferedInputStream(InflaterInputStream(ByteArrayInputStream(data), Inflater())))
			reader.skipBytes(3)

			val result = Object2DArray.nulls<MutableCell>(CHUNK_SIZE, CHUNK_SIZE)

			for (y in 0 until CHUNK_SIZE) {
				for (x in 0 until CHUNK_SIZE) {
					result[x, y] = MutableCell().read(reader)
				}
			}

			KOptional(result as Object2DArray<out AbstractCell>)
		}
	}

	override fun getEntities(pos: ChunkPos): CompletableFuture<KOptional<Collection<WorldObject>>> {
		return CompletableFuture.supplyAsync {
			val chunkX = pos.x
			val chunkY = pos.y
			val key = byteArrayOf(2, (chunkX shr 8).toByte(), chunkX.toByte(), (chunkY shr 8).toByte(), chunkY.toByte())
			val data = db.read(key) ?: return@supplyAsync KOptional.empty()

			val reader = DataInputStream(BufferedInputStream(InflaterInputStream(ByteArrayInputStream(data), Inflater())))
			val i = reader.readVarInt()
			val objects = ArrayList<WorldObject>()

			for (i2 in 0 until i) {
				val obj = VersionedJson(reader)

				if (obj.identifier == "ObjectEntity") {
					try {
						objects.add(WorldObject.fromJson(obj.content.asJsonObject))
					} catch (err: Throwable) {
						LOGGER.error("Unable to deserialize entity in chunk $pos", err)
					}
				} else {
					LOGGER.error("Unknown entity type in chunk $pos: ${obj.identifier}")
				}
			}

			return@supplyAsync KOptional(objects)
		}
	}

	companion object {
		private val vectors by lazy { Starbound.gson.getAdapter(Vector2i::class.java) }
		private val LOGGER = LogManager.getLogger()
	}
}
